﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Vehicles
{
    class Car: Vehicle
    {
        public Car(int NumWheels, int EngineCC, int NumPassengers, int KmPerLtr, int TankCapacity, FUEL_TYPE FuelType, int Fuel, int TopSpeed, string Manufacturer, string Model, int Year, int Price, string LicensePlate) :
            base( NumWheels,  EngineCC,  NumPassengers,  KmPerLtr,  TankCapacity,  FuelType,  Fuel,  TopSpeed,  Manufacturer,  Model,  Year,  Price, LicensePlate)
        {
            if (Fuel == 0)
                this.Fuel = this.TankCapacity;
            this.Odometer = 0;
        }

        public virtual string ToString()
        {
            return string.Format("{0}: {1}", this.GetType().Name, base.ToString()); 
        }
        public override string VirtualFunc()
        {
            return string.Format("Car: VirtualFunc()");
        }
        public new string VirtualFunc_New()
        {
            return string.Format("Car: VirtualFunc_New()");
        }

        // Not legal to override a non Virtual function
        public string NonVirtualFunc()
        {
           return string.Format("Car: NonVirtualFunc()");
        }

        // but declaring the function as NEW is OK
        public string VirtualFunc_Nothing()
        {
            return string.Format("Car: NonVirtualFunc_Nothing()");
        }


        /// <summary>
        /// Explanation for invalid access to PROTECTED properties of object of the same class.
        /// </summary>
        /// <param name="Other">another vehicle for comparison</param>
        /// <returns>
        ///     (-1) - this is faster; 
        ///     0 - equally fast; 
        ///     1 - other is faster.
        /// </returns>
        public int IsFaster(Object Other)
        {
            if (Other == null)
                return 1;

            Vehicle v = Other as Vehicle;
            if (v == null)
                return 1;

            // Polymorphism compilation error
            // The following is a syntax error! Won't compile, even when TopSpeed is a property that is PROTECTED!  
            //
            // return this.TopSpeed.CompareTo(v.TopSpeed);
            //
            // This is so becasue:
            // We are inside the code of a Car, and not inside the code of'Other'  
            // We CAN access the property TopSpeed of 'this'
            // but have no access to other.TopSpeed, 'other' is not on the inherted path!!
            // So we must use the Get/Set properties to access PROTECTED properties even when 
            // we are inside the same class.
            // There are two solutions:
            //      1. Use Get Set
            //      2. Make the property internal (see code below for isOlder(Object Other))

            return this.TopSpeed.CompareTo(v.GetTopSpeed());
        }


        /// <summary>
        /// Similar to IsFaster but accessing the Property Year of Vehicle which is declared as internal
        /// So we can access it directly without the Getter/Setter function.
        /// Note that Internal still does not allow 'public' access!
        /// </summary>
        /// <param name="Other">Another Vehicle to compare too</param>
        /// <returns>
        ///     (-1) - this is newer; 
        ///     0 - same age; 
        ///     1 - other is newer.
        /// </returns>
        public int IsNewer(Object Other)
        {
            if (Other == null)
                return 1;

            Vehicle v = Other as Vehicle;
            if (Other == null)
                return 1;

            // Here we can access v.Year because Vehicle declared it as 'internal'.
            return (this.Year.CompareTo(v.Year));
        }

        /// <summary>
        /// Price Base comparison
        /// Cannot do another type of comparison, so we must use IComparer:Compare 
        /// to add additional comparison capabilities like by speed.
        /// </summary>
        /// <param name="obj">Other vehicle to compare to</param>
        /// <returns></returns>
        public override int CompareTo(object obj)
        {
            if (obj == null)
                return 1;
            Vehicle v = obj as Vehicle;
            if (v == null)
                return 1;

            return this.Price.CompareTo(v.GetPrice());
        }
        // One example of Equals
        //public override bool Equals(object obj)
        //{
        //    Vehicle v = obj as Vehicle;
        //    if (v == null)
        //        return false;
        //    else
        //        return this.LicensePlate == v.GetLicensePlate();
        //}
        public override bool Equals(object obj)
        {
            Vehicle v = obj as Vehicle;
            if (v == null)
                return false;
            
            if (this != v)
                return false;

            return true;

        }

    }
}
