﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Vehicles
{
    abstract class Vehicle: IMove, IComparable
    {
        public enum FUEL_TYPE { DIESEL, ELECTRIC, HYBRID, PETROL, PROPANE};

        protected int NumWheels;
        protected int EngineCC;
        protected int NumPassengers;
        protected int KmPerLtr;
        protected int TankCapacity;
        protected FUEL_TYPE FuelType;
        protected int Fuel;
        protected int TopSpeed;
        //protected int Range;  // a calculated value
        protected string Manufacturer;
        protected string Model;
        internal int Year; // see Car.isOlder() for explanation for access to base class member: protected, internal
        protected int Price;
        protected int Odometer;
        protected string LicensePlate;

        protected Vehicle(int NumWheels, int EngineCC, int NumPassengers, int KmPerLtr, int TankCapacity, FUEL_TYPE FuelType, int Fuel, int TopSpeed, string Manufacturer, string Model, int Year, int Price, string LicensePlate)
        {
            this.NumWheels = NumWheels; 
            this.EngineCC = EngineCC;
            this.NumPassengers = NumPassengers;
            this.KmPerLtr = KmPerLtr;
            this.TankCapacity = TankCapacity;
            this.FuelType = FuelType;
            this.Fuel = Fuel;
            this.TopSpeed = TopSpeed;
            this.Manufacturer = Manufacturer;
            this.Model = Model;
            this.Year = Year;
            this.Price = Price;
            this.Odometer = 0;
            this.LicensePlate = LicensePlate;
    }

        public int GetNumWheel()
        {
            return NumWheels;
        }
        public void SetNumWheels(int NumWheels)
        {
            this.NumWheels = NumWheels;
        }

        public int GetEngineCC()
        {
            return EngineCC;
        }
        public void SetEngineCC(int EngineCC)
        {
            this.EngineCC = EngineCC;
        }
        public int GetNumPassengers()
        {
            return NumPassengers;
        }
        public void SetNumPassengers(int NumPassengers)
        {
            this.NumPassengers = NumPassengers;
        }
        public int GetKmPerLtr()
        {
            return KmPerLtr;
        }
        public void SetKmPerLtr(int KmPerLtr)
        {
            this.KmPerLtr = KmPerLtr;
        }
        public int GetTankCapacity()
        {
            return this.TankCapacity;
        }
        public void SetTankCapacity(int TankCapacity)
        {
            this.TankCapacity = TankCapacity;
        }
        public FUEL_TYPE GetFuelType()
        {
            return FuelType;
        }
        public void SetFuelType(FUEL_TYPE FuelType)
        {
            this.FuelType = FuelType;
        }
        public int GetFuel()
        {
            return Fuel;
        }
        public void SetFuel(int Fuel)
        {
            this.Fuel = Fuel;
        }
        public int GetTopSpeed()
        {
            return TopSpeed;
        }
        public void SetTopSpeed(int TopSpeed)
        {
            this.TopSpeed = TopSpeed;
        }
        public int GetRange()
        {
            return Fuel * KmPerLtr;
        }
        // no SetRange();

        public string GetManufacturer()
        {
            return Manufacturer;
        }
        public void SetManufacturer(string Manufacturer)
        {
            this.Manufacturer = Manufacturer;
        }
        public string GetModel()
        {
            return Model;
        }
        public void SetModel(string Model)
        {
            this.Model = Model;
        }
        public int GetYear()
        {
            return Year;
        }
        public void SetYear(int Year)
        {
            this.Year = Year;
        }
        public int GetPrice()
        {
            return Price;
        }
        public void SetPrice(int Price)
        {
            this.Price = Price;
        }
        public int GetOdometer()
        {
            return Odometer;
        }
        public string GetLicensePlate()
        {
            return LicensePlate;
        }
        /// <summary>
        /// Moves km kilometer. Consumes the amount of fuel for the driving
        /// Updates odometer accordinging.
        /// </summary>
        /// <param name="km">number of km wanting to drive</param>
        /// <returns>number km travelled which could be less than the km requested because not enough fuel for the drive.</returns>
        virtual public int Move(int km)
        {
            int maxTravel = Fuel * KmPerLtr;
            if (km <= maxTravel)
            {
                Odometer += km;
                Fuel -= (km / KmPerLtr);
                return km;
            }
            Odometer += maxTravel;
            Fuel -= maxTravel / KmPerLtr;
            return maxTravel;
        }
        public void FillTank(int fuel=int.MaxValue)
        {
            if (fuel < 0)
                return;

            this.Fuel = Math.Min(this.TankCapacity, this.GetFuel() + fuel);
        }
        public override string ToString()
        {
            return string.Format("Model: {1}, Manufacturer: {2}, Year: {3}, Price: ${4}, Fuel: {5} Kilometers: {6} LicensePlate {7}", this.GetType().Name, Model, Manufacturer, Year, Price, Fuel, Odometer, LicensePlate);
        }

        public virtual string VirtualFunc()
        {
            return string.Format("Vehicle: VirtualFunc()");
        }

        public string NonVirtualFunc()
        {
            return string.Format("Vehicle: NonVirtualFunc()");
        }
        public virtual string VirtualFunc_New()
        {
            return string.Format("Vehicle: VirtualFunc_New()");
        }
        public string VirtualFunc_Nothing()
        {
            return string.Format("Vehicle: VirtualFunc_Nothing");
        }

        public abstract int CompareTo(object obj);
    }
}
